/***********************************************************
*                       pC/SFS 1.14b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                    SFS_LLdriver_SPI.c                    *
*                    lowlevel SPI driver                   *
*             (to split lowlevel & HW specific)            *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../../inc/pC_SFS.h"


#define SFSPort_cmd_WREN     0x06        // write-enable
#define SFSPort_cmd_WRDI     0x04        // write-disable
#define SFSPort_cmd_RDSR     0x05        // read-status
#define SFSPort_cmd_WRSR     0x01        // write-status
#define SFSPort_cmd_READ     0x03        // read-data
#define SFSPort_cmd_WRITE    0x02        // write-data

#define SFSPort_cmd_SLEEP    0xB9        // sleep on MR2xR40, CY15B104Q, MB85AS4MT and FM25H20
#define SFSPort_cmd_WAKE     0xAB        // wake-up on MR2xR40

#define SFSPort_cmd_FSTRD    0x0B        // fast-read on CY15B104Q
#define SFSPort_cmd_RDID     0x9F        // read-ID   on CY15B104Q (9 bytes: 7F 7F 7F 7F 7F 7F C2 26 08)
                                         //           on MB85AS4MT (4 byte:  04 7F C9 03)

#define SFSPort_cmd_STORE    0x3C        // store on CY14B101Q
#define SFSPort_cmd_RECALL   0x60        // recall on CY14B101Q
#define SFSPort_cmd_ASENB    0x59        // auto-store-enable on CY14B101Q
#define SFSPort_cmd_ASDISB   0x3C        // auto-store-disable on CY14B101Q

#define SFSPort_status_def   0x40        // on FM25H20 - plus following bit-definitions
#define SFSPort_status_WPEN  (1 << 7)
#define SFSPort_status_BP1   (1 << 3)
#define SFSPort_status_BP0   (1 << 2)
#define SFSPort_status_WEL   (1 << 1)
#define SFSPort_status_RDY   (1 << 0)    // ready on CY14B101Q


/*
************************************************************
*                INIT SPI-INTERFACE & xRAM
************************************************************
*/
U08 SFSPort_Init(void)
{
    U08 sr = 0xFF;

    _sfsPort_Init();

    do {
        _sfsPort_CS_en();
        _sfsPort_RW(SFSPort_cmd_WREN);
        _sfsPort_CS_dis();
        sr = _sfsPort_readStatus(SFSPort_cmd_RDSR);
    } while ((sr & 0x8F) != SFSPort_status_WEL);

#ifdef SFS_MEM_CY15B104Q
    {
        U08 tmp[10], i;
        tmp[0] = 0;
        sfsPort_lock();
        _sfsPort_CS_en();
        _sfsPort_RW(SFSPort_cmd_RDID);
        i = 0;
        while(i < 9) tmp[i++] = _sfsPort_RW(0xFF);
        _sfsPort_CS_dis();
        sfsPort_unlock();
        if(tmp[8] != 0x08)
            while(1) _NOP();
    }
#endif

    return(SFS_NO_ERR);
}

/*
************************************************************
*                    READ FROM xRAM
************************************************************
*/
U08 SFSPort_Read(U32 offs, U08 OS_HUGE *dest, SFS_LONG length) KC51_REENTRANT
{
    SFS_LONG i = 0;

    sfsPort_lock();
    _sfsPort_CS_en();
    _sfsPort_RW(SFSPort_cmd_READ);
#if (SFS_MEM_HW_MEMSIZE_KB > 64)
    _sfsPort_RW(offs >> 16);
#endif
    _sfsPort_RW(offs >>  8);
    _sfsPort_RW(offs);
    i = 0;
    while(i < length) dest[i++] = _sfsPort_RW(0xFF);
    _sfsPort_CS_dis();
    sfsPort_unlock();
    return(SFS_NO_ERR);
}

/*
************************************************************
*         WRITE ALL OR PAGE INTO xRAM (internal)
************************************************************
*/
static void sfsPortWriteP(U32 offs, U08 OS_HUGE *source, SFS_LONG length) KC51_REENTRANT
{
    SFS_LONG i = 0;

    sfsPort_lock();
#if (SFS_MEM_HW_WREN != 0)
    _sfsPort_CS_en();
    _sfsPort_RW(SFSPort_cmd_WREN);
    _sfsPort_CS_dis();
#endif
    _sfsPort_CS_en();
    _sfsPort_RW(SFSPort_cmd_WRITE);
#if (SFS_MEM_HW_MEMSIZE_KB > 64)
    _sfsPort_RW(offs >> 16);
#endif
    _sfsPort_RW(offs >>  8);
    _sfsPort_RW(offs);
    i = 0;
    while(i < length) _sfsPort_RW(source[i++]);
    _sfsPort_CS_dis();
    sfsPort_unlock();
#if (SFS_MEM_HW_PAGESIZE > 1)
    while(_sfsPort_readStatus(SFSPort_cmd_RDSR) & SFSPort_status_RDY);      // not required on Everspin MRAMs
#endif
}

/*
************************************************************
*                    WRITE INTO xRAM
************************************************************
*/
U08 SFSPort_Write(U32 offs, U08 OS_HUGE *source, SFS_LONG length) KC51_REENTRANT
{
#if (SFS_MEM_HW_PAGESIZE > 1)
    SFS_LONG s;

    while(length > 0) {
        s = (((offs & (SFS_MEM_HW_PAGESIZE - 1)) + length) > SFS_MEM_HW_PAGESIZE)? (SFS_MEM_HW_PAGESIZE - (offs & (SFS_MEM_HW_PAGESIZE - 1))) : length;
        sfsPortWriteP(offs, source, s);
        source = &source[s];
        offs   += s;
        length -= s;
    }                                                      // up to page-size and read flag
#else
    sfsPortWriteP(offs, source, length);
#endif
    return(SFS_NO_ERR);
}

/*
************************************************************
*             FILL ALL OR PAGE INTO xRAM (internal)
************************************************************
*/
static void sfsPortFillP(U32 offs, U08 value, SFS_LONG length) KC51_REENTRANT
{
    SFS_LONG i = 0;

    sfsPort_lock();
#if (SFS_MEM_HW_WREN != 0)
    _sfsPort_CS_en();
    _sfsPort_RW(SFSPort_cmd_WREN);
    _sfsPort_CS_dis();
#endif
    _sfsPort_CS_en();
    _sfsPort_RW(SFSPort_cmd_WRITE);
#if (SFS_MEM_HW_MEMSIZE_KB > 64)
    _sfsPort_RW(offs >> 16);
#endif
    _sfsPort_RW(offs >>  8);
    _sfsPort_RW(offs);
    i = 0;
    while(i++ < length) _sfsPort_RW(value);
    _sfsPort_CS_dis();
    sfsPort_unlock();
#if (SFS_MEM_HW_PAGESIZE > 1)
    while(_sfsPort_readStatus(SFSPort_cmd_RDSR) & SFSPort_status_RDY);      // not required on Everspin MRAMs
#endif
}

/*
************************************************************
*                        FILL xRAM
************************************************************
*/
U08 SFSPort_Fill(U32 offs, U08 value, SFS_LONG length) KC51_REENTRANT
{
#if (SFS_MEM_HW_PAGESIZE > 1)
    SFS_LONG s;

    while(length > 0) {
        s = (((offs & (SFS_MEM_HW_PAGESIZE - 1)) + length) > SFS_MEM_HW_PAGESIZE)? (SFS_MEM_HW_PAGESIZE - (offs & (SFS_MEM_HW_PAGESIZE - 1))) : length;
        sfsPortFillP(offs, value, s);
        offs   += s;
        length -= s;
    }
#else
    sfsPortFillP(offs, value, length);
#endif
    return(SFS_NO_ERR);
}

/************************* END ****************************/
